<?php

namespace Ebuy\Questions\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Ebuy\Questions\Forms\Question as QuestionForm;
use Ebuy\Questions\Http\Requests\QuestionRequest;
use Ebuy\Questions\Interfaces\QuestionRepositoryInterface;
use Ebuy\Questions\Repositories\Eloquent\Filters\QuestionResourceFilter;
use Ebuy\Questions\Repositories\Eloquent\Presenters\QuestionListPresenter;

/**
 * Resource controller class for question.
 */
class QuestionResourceController extends BaseController
{

    /**
     * Initialize question resource controller.
     *
     *
     * @return null
     */
    public function __construct(QuestionRepositoryInterface $question)
    {
        parent::__construct();
        $this->form = QuestionForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('ebuy.questions.modules'), 'questions', guard_url('questions'));
        $this->repository = $question;
    }

    /**
     * Display a list of question.
     *
     * @return Response
     */
    public function index(QuestionRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(QuestionResourceFilter::class)
            ->setPresenter(QuestionListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('questions::question.names'))
            ->view('questions::question.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display question.
     *
     * @param Request $request
     * @param Model   $question
     *
     * @return Response
     */
    public function show(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('questions::question.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('questions::question.show')
            ->output();
    }

    /**
     * Show the form for creating a new question.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('questions::question.name'))
            ->view('questions::question.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new question.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('questions::question.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('questions/question/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/questions/question'))
                ->redirect();
        }

    }

    /**
     * Show question for editing.
     *
     * @param Request $request
     * @param Model   $question
     *
     * @return Response
     */
    public function edit(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('questions::question.name'))
            ->view('questions::question.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the question.
     *
     * @param Request $request
     * @param Model   $question
     *
     * @return Response
     */
    public function update(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('questions::question.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('questions/question/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('questions/question/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the question.
     *
     * @param Model   $question
     *
     * @return Response
     */
    public function destroy(QuestionRequest $request, QuestionRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('questions::question.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('questions/question/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('questions/question/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
